@@ -16,9 +16,13 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#define _GNU_SOURCE
+#define _LGPL_SOURCE
 #define __USE_LINUX_IOCTL_DEFS
 #include <sys/ioctl.h>
 #include <string.h>
+#include <common/align.h>
+#include <errno.h>
 
 #include "kernel-ctl.h"
 #include "kernel-ioctl.h"
@@ -108,6 +112,7 @@ int kernctl_create_channel(int fd, struct lttng_channel_attr *chops)
 {
 	struct lttng_kernel_channel channel;
 
+	memset(&channel, 0, sizeof(channel));
 	if (lttng_kernel_use_old_abi) {
 		struct lttng_kernel_old_channel old_channel;
 
@@ -139,6 +144,72 @@ int kernctl_create_channel(int fd, struct lttng_channel_attr *chops)
 	return ioctl(fd, LTTNG_KERNEL_CHANNEL, &channel);
 }
 
+int kernctl_syscall_mask(int fd, char **syscall_mask, uint32_t *nr_bits)
+{
+	struct lttng_kernel_syscall_mask kmask_len, *kmask = NULL;
+	size_t array_alloc_len;
+	char *new_mask;
+	int ret = 0;
+
+	if (!syscall_mask) {
+		ret = -1;
+		goto end;
+	}
+
+	if (!nr_bits) {
+		ret = -1;
+		goto end;
+	}
+
+	kmask_len.len = 0;
+	ret = ioctl(fd, LTTNG_KERNEL_SYSCALL_MASK, &kmask_len);
+	if (ret) {
+		goto end;
+	}
+
+	array_alloc_len = ALIGN(kmask_len.len, 8) >> 3;
+
+	kmask = zmalloc(sizeof(*kmask) + array_alloc_len);
+	if (!kmask) {
+		ret = -1;
+		goto end;
+	}
+
+	kmask->len = kmask_len.len;
+	ret = ioctl(fd, LTTNG_KERNEL_SYSCALL_MASK, kmask);
+	if (ret) {
+		goto end;
+	}
+
+	new_mask = realloc(*syscall_mask, array_alloc_len);
+	if (!new_mask) {
+		ret = -1;
+		goto end;
+	}
+	memcpy(new_mask, kmask->mask, array_alloc_len);
+	*syscall_mask = new_mask;
+	*nr_bits = kmask->len;
+
+end:
+	free(kmask);
+	return ret;
+}
+
+int kernctl_track_pid(int fd, int pid)
+{
+	return ioctl(fd, LTTNG_KERNEL_SESSION_TRACK_PID, pid);
+}
+
+int kernctl_untrack_pid(int fd, int pid)
+{
+	return ioctl(fd, LTTNG_KERNEL_SESSION_UNTRACK_PID, pid);
+}
+
+int kernctl_list_tracker_pids(int fd)
+{
+	return ioctl(fd, LTTNG_KERNEL_SESSION_LIST_TRACKER_PIDS);
+}
+
 int kernctl_create_stream(int fd)
 {
 	return compat_ioctl_no_arg(fd, LTTNG_KERNEL_OLD_STREAM,
@@ -229,12 +300,36 @@ int kernctl_stop_session(int fd)
 			LTTNG_KERNEL_SESSION_STOP);
 }
 
+int kernctl_filter(int fd, struct lttng_filter_bytecode *filter)
+{
+	struct lttng_kernel_filter_bytecode *kb;
+	uint32_t len;
+	int ret;
+
+	/* Translate bytecode to kernel bytecode */
+	kb = zmalloc(sizeof(*kb) + filter->len);
+	if (!kb)
+		return -ENOMEM;
+	kb->len = len = filter->len;
+	kb->reloc_offset = filter->reloc_table_offset;
+	kb->seqnum = filter->seqnum;
+	memcpy(kb->data, filter->data, len);
+	ret = ioctl(fd, LTTNG_KERNEL_FILTER, kb);
+	free(kb);
+	return ret;
+}
+
 int kernctl_tracepoint_list(int fd)
 {
 	return compat_ioctl_no_arg(fd, LTTNG_KERNEL_OLD_TRACEPOINT_LIST,
 			LTTNG_KERNEL_TRACEPOINT_LIST);
 }
 
+int kernctl_syscall_list(int fd)
+{
+	return ioctl(fd, LTTNG_KERNEL_SYSCALL_LIST);
+}
+
 int kernctl_tracer_version(int fd, struct lttng_kernel_tracer_version *v)
 {
 	int ret;
@@ -265,6 +360,12 @@ end:
 	return ret;
 }
 
+int kernctl_tracer_abi_version(int fd,
+		struct lttng_kernel_tracer_abi_version *v)
+{
+	return ioctl(fd, LTTNG_KERNEL_TRACER_ABI_VERSION, v);
+}
+
 int kernctl_wait_quiescent(int fd)
 {
 	return compat_ioctl_no_arg(fd, LTTNG_KERNEL_OLD_WAIT_QUIESCENT,
